import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import ode

EPS=1.e-4
#x=(pi G M f)^(2/3)=v^2

def energy(x,eta):
    return -0.5*eta*x*x

def den_dx(x,eta):
    return -eta*x

def flux(x,eta):
    return 32./5.*eta*eta*x*x*x*x*x

def tbyM_x(x,eta):
    return -energy(x,eta)/flux(x,eta)

def dx_dt(Mt,x,eta):
    return -flux(x,eta)/den_dx(x,eta)

def int_x(x0,eta):

    x_max = 1.
    solver = ode(dx_dt)
    solver.set_integrator('dopri5')
    solver.set_initial_value(x0, 0)
    solver.set_f_params(eta)
    tbyM = [0]
    x_tM = [x0]
    dtbyM=1.e3
    while solver.successful() and solver.y < x_max:
        solver.integrate(solver.t+dtbyM)
        tbyM.append(solver.t)
        x_tM.append(solver.y)
        if x_tM[-1]<x_tM[-2] or x_tM[-1]>x_max:
            dtbyM*=0.9
            del x_tM[-1]
            del tbyM[-1]
        if dtbyM<EPS:
            break
        #print(solver.y)
    return np.array(tbyM), np.array(x_tM)

x0=1.e-2
eta=0.25
print("t/M ends at ",tbyM_x(x0,eta))
tbyM,x_tM=int_x(x0,eta)
print("For x[t0]=",x0,"  eta = ",eta,"  I get ",len(tbyM)," ",len(x_tM)," un-equally spaced points")

plt.figure()
plt.plot(tbyM,x_tM,marker="x")
plt.xlabel("t/(GM)")
plt.ylabel("x")
plt.show()
